module fortran_procedure
  use iso_c_binding
  implicit none
  public :: foo
  public :: fdata_pointer, fdata_allocate, fdata_deallocate
  public :: data_write_f
  public :: pdata_recieve, pdata_manipulate
  real(8), pointer, dimension(:, :, :) :: data

contains

  subroutine foo(bar) bind(C, name='foo')
    use iso_c_binding
    integer(c_int), value :: bar

    write(*, *) bar
  end subroutine foo

  function fdata_pointer(ni, nj, nk) bind(C, name='fdata_pointer')
    type(c_ptr) :: fdata_pointer
    integer(c_int) :: ni, nj, nk
    integer, dimension(3) :: ds

    ds = shape(data)
    ni = ds(1); nj = ds(2); nk = ds(3)
    fdata_pointer = c_loc(data(1, 1, 1))
  end function fdata_pointer

  subroutine fdata_allocate() bind(C, name='fdata_allocate')
    integer :: ni, nj, nk
    integer :: i, j, k
    integer, dimension(3) :: ds

    ni = 3; nj = 4; nk = 2
    allocate(data(ni, nj, nk))
    ds = shape(data)
    do k = 1, ds(3)
       do j = 1, ds(2)
          do i = 1, ds(1)
             data(i,j,k) = k * 100 + j * 10 + i
          end do
       end do
    end do
  end subroutine fdata_allocate

  subroutine data_write_f() bind(C, name='data_write_f')
    integer :: i, j, k
    integer, dimension(3) :: ds

    write(*, *) 'in Fortran'
    ds = shape(data)
    do k = 1, ds(3)
       do j = 1, ds(2)
          do i = 1, ds(1)
             write(*, *) i, j, k, data(i, j, k)
          end do
       end do
    end do
  end subroutine data_write_f

  subroutine fdata_deallocate() bind(C, name='fdata_deallocate')
    deallocate(data)
    nullify(data)
  end subroutine fdata_deallocate

  subroutine pdata_recieve(ni, nj, nk, pdata) bind(C, name='pdata_recieve')
    integer(c_int), intent(in) :: ni, nj, nk
    real(c_double), target :: pdata(ni, nj, nk)

    data => pdata
  end subroutine pdata_recieve

  subroutine pdata_manipulate() bind(C, name='pdata_manipulate')
    integer :: i, j, k

    i = 2; j = 3; k = 1
    data(i, j, k) = 2021
  end subroutine pdata_manipulate

end module fortran_procedure

